<?php
/**
 * Plugin Name: WPC Product Table for WooCommerce
 * Plugin URI: https://wpclever.net/
 * Description: WPC Product Table helps you show selected products as a table on the page.
 * Version: 2.0.0
 * Author: WPClever
 * Author URI: https://wpclever.net
 * Text Domain: wpc-product-table
 * Domain Path: /languages/
 * Requires at least: 4.0
 * Tested up to: 6.0
 * WC requires at least: 3.0
 * WC tested up to: 6.5
 */

defined( 'ABSPATH' ) || exit;

! defined( 'WPCPT_VERSION' ) && define( 'WPCPT_VERSION', '2.0.0' );
! defined( 'WPCPT_FILE' ) && define( 'WPCPT_FILE', __FILE__ );
! defined( 'WPCPT_URI' ) && define( 'WPCPT_URI', plugin_dir_url( __FILE__ ) );
! defined( 'WPCPT_DIR' ) && define( 'WPCPT_DIR', plugin_dir_path( __FILE__ ) );
! defined( 'WPCPT_REVIEWS' ) && define( 'WPCPT_REVIEWS', 'https://wordpress.org/support/plugin/wpc-product-table/reviews/?filter=5' );
! defined( 'WPCPT_CHANGELOG' ) && define( 'WPCPT_CHANGELOG', 'https://wordpress.org/plugins/wpc-product-table/#developers' );
! defined( 'WPCPT_DISCUSSION' ) && define( 'WPCPT_DISCUSSION', 'https://wordpress.org/support/plugin/wpc-product-table' );
! defined( 'WPC_URI' ) && define( 'WPC_URI', WPCPT_URI );

include 'includes/wpc-dashboard.php';
include 'includes/wpc-menu.php';
include 'includes/wpc-kit.php';

if ( ! function_exists( 'wpcpt_init' ) ) {
	add_action( 'plugins_loaded', 'wpcpt_init', 11 );

	function wpcpt_init() {
		// load text-domain
		load_plugin_textdomain( 'wpc-product-table', false, basename( __DIR__ ) . '/languages/' );

		if ( ! function_exists( 'WC' ) || ! version_compare( WC()->version, '3.0', '>=' ) ) {
			add_action( 'admin_notices', 'wpcpt_notice_wc' );

			return;
		}

		if ( ! class_exists( 'WPCleverWpcpt' ) ) {
			class WPCleverWpcpt {
				protected static $columns = array();
				protected static $localization = array();
				protected static $instance;

				public static function instance() {
					if ( ! isset( self::$instance ) ) {
						self::$instance = new self();
						self::$instance->init();
					}

					return self::$instance;
				}

				function init() {
					add_action( 'init', array( $this, 'wp_init' ) );
					add_action( 'admin_menu', array( $this, 'admin_menu' ) );
					add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ) );
					add_action( 'save_post_wpc_product_table', array( $this, 'save_product_table' ) );
					add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
					add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );

					// action links
					add_filter( 'plugin_action_links', array( $this, 'action_links' ), 10, 2 );
					add_filter( 'plugin_row_meta', array( $this, 'row_meta' ), 10, 2 );

					// ajax
					add_action( 'wp_ajax_wpcpt_add_column', array( $this, 'add_column' ) );
					add_action( 'wp_ajax_wpcpt_search_term', array( $this, 'search_term' ) );

					// table columns
					add_filter( 'manage_edit-wpc_product_table_columns', array( $this, 'table_columns' ), 10 );
					add_action( 'manage_wpc_product_table_posts_custom_column', array(
						$this,
						'table_custom_column'
					), 10, 2 );

					self::$columns = array(
						'order'             => esc_html__( 'Order', 'wpc-product-table' ),
						'thumbnail'         => esc_html__( 'Thumbnail', 'wpc-product-table' ),
						'name'              => esc_html__( 'Name', 'wpc-product-table' ),
						'name_price'        => esc_html__( 'Name & Price', 'wpc-product-table' ),
						'price'             => esc_html__( 'Price', 'wpc-product-table' ),
						'price_html'        => esc_html__( 'Price HTML', 'wpc-product-table' ),
						'sku'               => esc_html__( 'SKU', 'wpc-product-table' ),
						'review'            => esc_html__( 'Review', 'wpc-product-table' ),
						'short_description' => esc_html__( 'Short Description', 'wpc-product-table' ),
						'add_to_cart'       => esc_html__( 'Add To Cart', 'wpc-product-table' ),
						'availability'      => esc_html__( 'Availability', 'wpc-product-table' ),
						'stock'             => esc_html__( 'Stock', 'wpc-product-table' ),
						'compare'           => esc_html__( 'Compare', 'wpc-product-table' ),
						'quickview'         => esc_html__( 'Quick View', 'wpc-product-table' ),
						'wishlist'          => esc_html__( 'Wishlist', 'wpc-product-table' ),
						'custom'            => esc_html__( 'Custom', 'wpc-product-table' ),
					);

					self::$localization = array(
						'decimal'        => '',
						'emptyTable'     => esc_html__( 'No data available in table', 'wpc-product-table' ),
						'info'           => esc_html__( 'Showing _START_ to _END_ of _TOTAL_ entries', 'wpc-product-table' ),
						'infoEmpty'      => esc_html__( 'Showing 0 to 0 of 0 entries', 'wpc-product-table' ),
						'infoFiltered'   => esc_html__( '(filtered from _MAX_ total entries)', 'wpc-product-table' ),
						'infoPostFix'    => '',
						'thousands'      => esc_html__( ',', 'wpc-product-table' ),
						'lengthMenu'     => esc_html__( 'Show _MENU_ entries', 'wpc-product-table' ),
						'loadingRecords' => esc_html__( 'Loading...', 'wpc-product-table' ),
						'processing'     => esc_html__( 'Processing...', 'wpc-product-table' ),
						'search'         => esc_html__( 'Search:', 'wpc-product-table' ),
						'zeroRecords'    => esc_html__( 'No matching records found', 'wpc-product-table' ),
						'paginate'       => array(
							'first'    => esc_html__( 'First', 'wpc-product-table' ),
							'last'     => esc_html__( 'Last', 'wpc-product-table' ),
							'next'     => esc_html__( 'Next', 'wpc-product-table' ),
							'previous' => esc_html__( 'Previous', 'wpc-product-table' )
						),
						'aria'           => array(
							'sortAscending'  => esc_html__( ': activate to sort column ascending', 'wpc-product-table' ),
							'sortDescending' => esc_html__( ': activate to sort column descending', 'wpc-product-table' )
						)
					);
				}


				function wp_init() {
					$labels = array(
						'name'          => _x( 'Product Tables', 'Post Type General Name', 'wpc-product-table' ),
						'singular_name' => _x( 'Product Table', 'Post Type Singular Name', 'wpc-product-table' ),
						'add_new_item'  => esc_html__( 'Add New Product Table', 'wpc-product-table' ),
						'add_new'       => esc_html__( 'Add New', 'wpc-product-table' ),
						'edit_item'     => esc_html__( 'Edit Product Table', 'wpc-product-table' ),
						'update_item'   => esc_html__( 'Update Product Table', 'wpc-product-table' ),
						'search_items'  => esc_html__( 'Search Product Table', 'wpc-product-table' ),
					);

					$args = array(
						'label'               => esc_html__( 'Product Table', 'wpc-product-table' ),
						'labels'              => $labels,
						'supports'            => array( 'title', 'excerpt' ),
						'hierarchical'        => false,
						'public'              => false,
						'show_ui'             => true,
						'show_in_menu'        => true,
						'show_in_nav_menus'   => true,
						'show_in_admin_bar'   => true,
						'menu_position'       => 28,
						'menu_icon'           => 'dashicons-list-view',
						'can_export'          => true,
						'has_archive'         => false,
						'exclude_from_search' => true,
						'publicly_queryable'  => false,
						'capability_type'     => 'post',
						'show_in_rest'        => false,
					);

					register_post_type( 'wpc_product_table', $args );

					// shortcode
					add_shortcode( 'wpc_product_table', array( $this, 'shortcode' ) );
				}

				function admin_menu() {
					add_submenu_page( 'wpclever', esc_html__( 'WPC Product Table', 'wpc-product-table' ), esc_html__( 'Product Table', 'wpc-product-table' ), 'manage_options', 'wpclever-wpcpt', array(
						$this,
						'admin_menu_content'
					) );
				}

				function admin_menu_content() {
					add_thickbox();
					$active_tab = isset( $_GET['tab'] ) ? sanitize_key( $_GET['tab'] ) : 'settings';
					?>
                    <div class="wpclever_settings_page wrap">
                        <h1 class="wpclever_settings_page_title"><?php echo esc_html__( 'WPC Product Table', 'wpc-product-table' ) . ' ' . WPCPT_VERSION; ?></h1>
                        <div class="wpclever_settings_page_desc about-text">
                            <p>
								<?php printf( esc_html__( 'Thank you for using our plugin! If you are satisfied, please reward it a full five-star %s rating.', 'wpc-product-table' ), '<span style="color:#ffb900">&#9733;&#9733;&#9733;&#9733;&#9733;</span>' ); ?>
                                <br/>
                                <a href="<?php echo esc_url( WPCPT_REVIEWS ); ?>"
                                   target="_blank"><?php esc_html_e( 'Reviews', 'wpc-product-table' ); ?></a> | <a
                                        href="<?php echo esc_url( WPCPT_CHANGELOG ); ?>"
                                        target="_blank"><?php esc_html_e( 'Changelog', 'wpc-product-table' ); ?></a>
                                | <a href="<?php echo esc_url( WPCPT_DISCUSSION ); ?>"
                                     target="_blank"><?php esc_html_e( 'Discussion', 'wpc-product-table' ); ?></a>
                            </p>
                        </div>
                        <div class="wpclever_settings_page_nav">
                            <h2 class="nav-tab-wrapper">
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpcpt&tab=how' ); ?>"
                                   class="<?php echo esc_attr( $active_tab === 'how' ? 'nav-tab nav-tab-active' : 'nav-tab' ); ?>">
									<?php esc_html_e( 'How to use?', 'wpc-product-table' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpcpt&tab=settings' ); ?>"
                                   class="<?php echo esc_attr( $active_tab === 'settings' ? 'nav-tab nav-tab-active' : 'nav-tab' ); ?>">
									<?php esc_html_e( 'Settings', 'wpc-product-table' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'edit.php?post_type=wpc_product_table' ); ?>"
                                   class="nav-tab">
									<?php esc_html_e( 'Product Tables', 'wpc-product-table' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-wpcpt&tab=premium' ); ?>"
                                   class="<?php echo esc_attr( $active_tab === 'premium' ? 'nav-tab nav-tab-active' : 'nav-tab' ); ?>"
                                   style="color: #c9356e">
									<?php esc_html_e( 'Premium Version', 'wpc-product-table' ); ?>
                                </a>
                                <a href="<?php echo admin_url( 'admin.php?page=wpclever-kit' ); ?>" class="nav-tab">
									<?php esc_html_e( 'Essential Kit', 'wpc-product-table' ); ?>
                                </a>
                            </h2>
                        </div>
                        <div class="wpclever_settings_page_content">
							<?php if ( $active_tab === 'how' ) { ?>
                                <div class="wpclever_settings_page_content_text">
                                    <p>
										<?php esc_html_e( 'Please go to WP-admin >> Product Tables to create a table then insert the shortcode where you want to show this table.', 'wpc-product-table' ); ?>
                                    </p>
                                </div>
							<?php } elseif ( $active_tab === 'settings' ) {
								$link_image = get_option( 'wpcpt_link_image', 'yes' );
								$link_name  = get_option( 'wpcpt_link_name', 'yes' );
								?>
                                <form method="post" action="options.php">
									<?php wp_nonce_field( 'update-options' ); ?>
                                    <table class="form-table">
                                        <tr>
                                            <th><?php esc_html_e( 'Link on the product image', 'wpc-product-table' ); ?></th>
                                            <td>
                                                <select name="wpcpt_link_image">
                                                    <option value="yes" <?php echo esc_attr( $link_image === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open in the same tab', 'wpc-product-table' ); ?>
                                                    </option>
                                                    <option value="yes_blank" <?php echo esc_attr( $link_image === 'yes_blank' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open in the new tab', 'wpc-product-table' ); ?>
                                                    </option>
                                                    <option value="yes_popup" <?php echo esc_attr( $link_image === 'yes_popup' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open quick view popup', 'wpc-product-table' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo esc_attr( $link_image === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-product-table' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description">If you choose "Open quick view popup", please install <a
                                                            href="<?php echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=woo-smart-quick-view&TB_iframe=true&width=800&height=550' ) ); ?>"
                                                            class="thickbox" title="Install WPC Smart Quick View">WPC Smart Quick View</a> to make it work.</span>
                                            </td>
                                        </tr>
                                        <tr>
                                            <th><?php esc_html_e( 'Link on the product name', 'wpc-product-table' ); ?></th>
                                            <td>
                                                <select name="wpcpt_link_name">
                                                    <option value="yes" <?php echo esc_attr( $link_name === 'yes' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open in the same tab', 'wpc-product-table' ); ?>
                                                    </option>
                                                    <option value="yes_blank" <?php echo esc_attr( $link_name === 'yes_blank' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open in the new tab', 'wpc-product-table' ); ?>
                                                    </option>
                                                    <option value="yes_popup" <?php echo esc_attr( $link_name === 'yes_popup' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'Yes, open quick view popup', 'wpc-product-table' ); ?>
                                                    </option>
                                                    <option value="no" <?php echo esc_attr( $link_name === 'no' ? 'selected' : '' ); ?>>
														<?php esc_html_e( 'No', 'wpc-product-table' ); ?>
                                                    </option>
                                                </select>
                                                <span class="description">If you choose "Open quick view popup", please install <a
                                                            href="<?php echo esc_url( admin_url( 'plugin-install.php?tab=plugin-information&plugin=woo-smart-quick-view&TB_iframe=true&width=800&height=550' ) ); ?>"
                                                            class="thickbox" title="Install WPC Smart Quick View">WPC Smart Quick View</a> to make it work.</span>
                                            </td>
                                        </tr>
                                        <tr class="submit">
                                            <th colspan="2">
                                                <input type="submit" name="submit" class="button button-primary"
                                                       value="<?php esc_html_e( 'Update Options', 'wpc-product-table' ); ?>"/>
                                                <input type="hidden" name="action" value="update"/>
                                                <input type="hidden" name="page_options"
                                                       value="wpcpt_link_image,wpcpt_link_name"/>
                                            </th>
                                        </tr>
                                    </table>
                                </form>
							<?php } elseif ( $active_tab === 'premium' ) { ?>
                                <div class="wpclever_settings_page_content_text">
                                    <p>
                                        Get the Premium Version just $29! <a
                                                href="https://wpclever.net/downloads/product-table?utm_source=pro&utm_medium=wpcpt&utm_campaign=wporg"
                                                target="_blank">https://wpclever.net/downloads/product-table</a>
                                    </p>
                                    <p><strong>Extra features for Premium Version:</strong></p>
                                    <ul style="margin-bottom: 0">
                                        <li>- Add columns with custom data.</li>
                                        <li>- Get the lifetime update & premium support.</li>
                                    </ul>
                                </div>
							<?php } ?>
                        </div>
                    </div>
					<?php
				}

				function action_links( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$how                  = '<a href="' . admin_url( 'admin.php?page=wpclever-wpcpt&tab=how' ) . '">' . esc_html__( 'How to use?', 'wpc-product-table' ) . '</a>';
						$settings             = '<a href="' . admin_url( 'admin.php?page=wpclever-wpcpt&tab=settings' ) . '">' . esc_html__( 'Settings', 'wpc-product-table' ) . '</a>';
						$links['wpc-premium'] = '<a href="' . admin_url( 'admin.php?page=wpclever-wpcpt&tab=premium' ) . '">' . esc_html__( 'Premium Version', 'wpc-product-table' ) . '</a>';
						array_unshift( $links, $how, $settings );
					}

					return (array) $links;
				}

				function row_meta( $links, $file ) {
					static $plugin;

					if ( ! isset( $plugin ) ) {
						$plugin = plugin_basename( __FILE__ );
					}

					if ( $plugin === $file ) {
						$row_meta = array(
							'support' => '<a href="' . esc_url( WPCPT_DISCUSSION ) . '" target="_blank">' . esc_html__( 'Community support', 'wpc-product-table' ) . '</a>',
						);

						return array_merge( $links, $row_meta );
					}

					return (array) $links;
				}

				function get_table_data( $id ) {
					$data                  = array( 'id' => $id );
					$data['type']          = get_post_meta( $id, 'type', true ) ?: 'products';
					$data['categories']    = get_post_meta( $id, 'categories', true ) ?: '';
					$data['tags']          = get_post_meta( $id, 'tags', true ) ?: '';
					$data['terms']         = get_post_meta( $id, 'terms', true ) ?: '';
					$data['include']       = get_post_meta( $id, 'include', true ) ?: '';
					$data['exclude']       = get_post_meta( $id, 'exclude', true ) ?: '';
					$data['limit']         = get_post_meta( $id, 'limit', true ) ?: 50;
					$data['orderby']       = get_post_meta( $id, 'orderby', true ) ?: 'default';
					$data['order']         = get_post_meta( $id, 'order', true ) ?: 'default';
					$data['style']         = get_post_meta( $id, 'style', true ) ?: 'default';
					$data['extra_classes'] = get_post_meta( $id, 'extra_classes', true ) ?: '';
					$data['columns']       = get_post_meta( $id, 'columns', true ) ?: '';
					$data['localization']  = get_post_meta( $id, 'localization', true ) ?: array();

					return $data;
				}

				function get_table_products( $data ) {
					$args = array(
						'is_wpcpt' => true,
						'limit'    => (int) $data['limit']
					);

					if ( ! empty( $data['orderby'] ) ) {
						$args['orderby'] = esc_attr( $data['orderby'] );
					}

					if ( ! empty( $data['order'] ) ) {
						$args['order'] = esc_attr( $data['order'] );
					}

					$cats    = explode( ',', $data['categories'] );
					$tags    = array_map( 'trim', explode( ',', $data['tags'] ) );
					$terms   = array_map( 'trim', explode( ',', $data['terms'] ) );
					$include = explode( ',', $data['include'] );
					$exclude = explode( ',', $data['exclude'] );

					switch ( $data['type'] ) {
						case 'best_selling':
							$args['meta_key'] = 'total_sales';
							$args['orderby']  = 'meta_value_num';
							$args['order']    = 'desc';

							break;
						case 'on_sale':
							$args['include'] = array_unique( array_merge( array( 0 ), wc_get_product_ids_on_sale() ) );

							break;
						case 'products':
							$args['type']    = array_merge( array_keys( wc_get_product_types() ), array( 'variation' ) );
							$args['include'] = array_unique( array_merge( array( 0 ), $include ) );
							$args['limit']   = - 1;

							if ( $args['orderby'] === 'default' ) {
								$args['orderby'] = 'include';
							}

							break;
						case 'categories_tags':
							if ( ! empty( $data['categories'] ) && ( $data['categories'] !== 'null' ) && ! empty( $cats ) ) {
								$args['category'] = array_unique( $cats );
							}

							if ( ! empty( $data['tags'] ) && ! empty( $tags ) ) {
								$args['tag'] = array_unique( $tags );
							}

							if ( ! empty( $data['exclude'] ) && ! empty( $exclude ) ) {
								$args['exclude'] = array_unique( $exclude );
							}

							break;
						default:
							// terms
							$args['tax_query'] = array(
								array(
									'taxonomy' => $data['type'],
									'field'    => 'slug',
									'terms'    => $terms,
									'operator' => 'IN',
								)
							);
					}

					return wc_get_products( $args );
				}

				function shortcode( $attrs ) {
					$output = '';
					$attrs  = shortcode_atts( array( 'id' => 0, 'name' => '' ), $attrs, 'wpc_product_table' );

					if ( $attrs['id'] ) {
						$data      = self::get_table_data( $attrs['id'] );
						$_products = self::get_table_products( $data );

						if ( $_products && ( count( $_products ) > 0 ) && is_array( $data['columns'] ) && ( count( $data['columns'] ) > 0 ) ) {
							foreach ( self::$localization as $key => $val ) {
								if ( is_array( $val ) ) {
									foreach ( $val as $k => $v ) {
										if ( ! isset( $data['localization'][ $key ][ $k ] ) || empty( $data['localization'][ $key ][ $k ] ) ) {
											$data['localization'][ $key ][ $k ] = $v;
										}
									}
								} else {
									if ( ! isset( $data['localization'][ $key ] ) || empty( $data['localization'][ $key ] ) ) {
										$data['localization'][ $key ] = $val;
									}
								}
							}

							$table_classes = 'display wpc_product_table wpc_product_table_style_' . ( ! empty( $data['style'] ) ? $data['style'] : 'default' ) . ( ! empty( $data['extra_classes'] ) ? ' ' . $data['extra_classes'] : '' );

							ob_start();
							?>
                            <div class="wpc_product_table_wrapper wpc_product_table_wrapper_<?php echo esc_attr( $attrs['id'] ); ?> wpc_product_table_wrapper_style_<?php echo esc_attr( ! empty( $data['style'] ) ? $data['style'] : 'default' ); ?>">
                                <table class="<?php echo esc_attr( $table_classes ); ?>"
                                       data-language="<?php echo htmlspecialchars( json_encode( $data['localization'] ), ENT_QUOTES, 'UTF-8' ); ?>">
                                    <thead>
                                    <tr class="wpc_product_table_tr">
										<?php
										foreach ( $data['columns'] as $column ) {
											$th_class = 'wpc_product_table_th wpc_product_table_th_' . esc_attr( $column['type'] );
											echo '<th class="' . $th_class . '" data-orderable="' . esc_attr( isset( $column['orderable'] ) ? $column['orderable'] : 'false' ) . '">' . esc_html( $column['title'] ) . '</th>';
										}
										?>
                                    </tr>
                                    </thead>
                                    <tbody>
									<?php
									$order = 1;

									foreach ( $_products as $_product ) {
										if ( ! $_product || ! $_product->is_visible() ) {
											continue;
										}

										$_product_id = $_product->get_id();
										$link_image  = get_option( 'wpcpt_link_image', 'yes' );
										$link_name   = get_option( 'wpcpt_link_name', 'yes' );
										?>
                                        <tr class="wpc_product_table_tr wpc_product_table_tr_<?php echo esc_attr( $_product_id ); ?>">
											<?php
											foreach ( $data['columns'] as $column ) {
												$td_class = 'wpc_product_table_td wpc_product_table_td_' . esc_attr( $column['type'] ) . ' wpc_product_table_td_align_' . esc_attr( isset( $column['align'] ) ? $column['align'] : 'left' ) . ' wpc_product_table_td_valign_' . esc_attr( isset( $column['valign'] ) ? $column['valign'] : 'top' );

												switch ( $column['type'] ) {
													case 'order':
														$td_content = '<td class="' . $td_class . '">' . $order . '</td>';

														break;
													case 'thumbnail':
														if ( $link_image !== 'no' ) {
															$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product_id ) . '"><a href="' . $_product->get_permalink() . '" ' . ( $link_image === 'yes_popup' ? 'class="woosq-link no-ajaxy" data-id="' . $_product_id . '"' : '' ) . ' ' . ( $link_image === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . $_product->get_image() . '</a></td>';
														} else {
															$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product_id ) . '">' . $_product->get_image() . '</td>';
														}

														break;
													case 'name':
														if ( $link_name !== 'no' ) {
															$td_content = '<td class="' . $td_class . '"><a href="' . $_product->get_permalink() . '" ' . ( $link_image === 'yes_popup' ? 'class="woosq-link no-ajaxy" data-id="' . $_product_id . '"' : '' ) . ' ' . ( $link_image === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . $_product->get_name() . '</a></td>';
														} else {
															$td_content = '<td class="' . $td_class . '">' . $_product->get_name() . '</td>';
														}

														break;
													case 'name_price':
														if ( $link_name !== 'no' ) {
															$td_content = '<td class="' . $td_class . '"><div class="wpcpt_product_name""><a href="' . $_product->get_permalink() . '" ' . ( $link_image === 'yes_popup' ? 'class="woosq-link no-ajaxy" data-id="' . $_product_id . '"' : '' ) . ' ' . ( $link_image === 'yes_blank' ? 'target="_blank"' : '' ) . '>' . $_product->get_name() . '</a></div><div class="wpcpt_product_price">' . $_product->get_price_html() . '</div></td>';
														} else {
															$td_content = '<td class="' . $td_class . '"><div class="wpcpt_product_name"">' . $_product->get_name() . '</div><div class="wpcpt_product_price">' . $_product->get_price_html() . '</div></td>';
														}

														break;
													case 'price':
														$td_content = '<td class="' . $td_class . '">' . $_product->get_price() . '</td>';

														break;
													case 'price_html':
														$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product->get_price() ) . '">' . $_product->get_price_html() . '</td>';

														break;
													case 'sku':
														$td_content = '<td class="' . $td_class . '">' . $_product->get_sku() . '</td>';

														break;
													case 'add_to_cart':
														$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product_id ) . '">' . do_shortcode( '[add_to_cart style="" show_price="false" id="' . $_product_id . '"]' ) . '</td>';

														break;
													case 'short_description':
														$td_content = '<td class="' . $td_class . '">' . $_product->get_short_description() . '</td>';

														break;
													case 'stock':
														$td_content = '<td class="' . $td_class . '">' . wc_get_stock_html( $_product ) . '</td>';

														break;
													case 'availability':
														$availability = $_product->get_availability();
														$td_content   = '<td class="' . $td_class . '">' . $availability['availability'] . '</td>';

														break;
													case 'review':
														$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product->get_average_rating() ) . '">' . wc_get_rating_html( $_product->get_average_rating() ) . '</td>';

														break;
													case 'compare':
														$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product_id ) . '">' . do_shortcode( '[woosc id="' . $_product_id . '"]' ) . '</td>';

														break;
													case 'quickview':
														$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product_id ) . '">' . do_shortcode( '[woosq id="' . $_product_id . '"]' ) . '</td>';

														break;
													case 'wishlist':
														$td_content = '<td class="' . $td_class . '" data-sort="' . esc_attr( $_product_id ) . '">' . do_shortcode( '[woosw id="' . $_product_id . '"]' ) . '</td>';

														break;
													default:
														$td_content = '<td class="' . $td_class . '">&nbsp;</td>';
												}

												echo apply_filters( 'wpcpt_' . $column['type'] . '_content', $td_content, $column, $_product );
											}
											?>
                                        </tr>
										<?php
										$order ++;
									}
									?>
                                    </tbody>
                                    <tfoot>
                                    <tr class="wpc_product_table_tr">
										<?php
										foreach ( $data['columns'] as $column ) {
											$th_class = 'wpc_product_table_th wpc_product_table_th_' . esc_attr( $column['type'] );
											echo '<th class="' . $th_class . '">' . esc_html( $column['title'] ) . '</th>';
										}
										?>
                                    </tr>
                                    </tfoot>
                                </table>
                            </div>
							<?php
							$output = ob_get_clean();
						}
					}

					return apply_filters( 'wpc_product_table', $output, $attrs['id'] );
				}

				function add_meta_boxes() {
					add_meta_box( 'wpcpt_shortcode', esc_html__( 'Shortcode', 'wpc-product-table' ), array(
						$this,
						'shortcode_callback'
					), 'wpc_product_table', 'advanced', 'high' );
					add_meta_box( 'wpcpt_configuration', esc_html__( 'Configuration', 'wpc-product-table' ), array(
						$this,
						'configuration_callback'
					), 'wpc_product_table', 'advanced', 'low' );
				}

				function shortcode_callback( $post ) {
					echo '<div class="wpcpt_shortcode_txt"><input type="text" class="wpcpt_shortcode_input" data-id="' . $post->ID . '" readonly value="[wpc_product_table id=&quot;' . $post->ID . '&quot; name=&quot;' . esc_attr( $post->post_title ) . '&quot;]"/></div>';
					echo '<div class="wpcpt_shortcode_des">' . esc_html__( 'Place above shortcode where you want to show this product table.', 'wpc-product-table' ) . '</div>';
				}

				function configuration_callback( $post ) {
					$post_id = $post->ID;
					$data    = self::get_table_data( $post_id );

					wp_enqueue_editor();
					?>
                    <h2 class="nav-tab-wrapper">
                        <a href="#" class="nav-tab nav-tab-active wpcpt_configuration_nav" data-tab="source">
							<?php esc_html_e( 'Source', 'wpc-product-table' ); ?>
                        </a>
                        <a href="#" class="nav-tab wpcpt_configuration_nav" data-tab="design">
							<?php esc_html_e( 'Design', 'wpc-product-table' ); ?>
                        </a>
                        <a href="#" class="nav-tab wpcpt_configuration_nav" data-tab="localization">
							<?php esc_html_e( 'Localization', 'wpc-product-table' ); ?>
                        </a>
                    </h2>
                    <div class="wpcpt_configuration wpcpt_configuration_source">
                        <table class="wpcpt_configuration_table">
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Type', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <select name="wpcpt_configuration_type" class="wpcpt_configuration_type">
                                        <option value="products" <?php echo esc_attr( $data['type'] === 'products' ? 'selected' : '' ); ?>><?php esc_html_e( 'Products', 'wpc-product-table' ); ?></option>
                                        <option value="categories_tags" <?php echo esc_attr( $data['type'] === 'categories_tags' ? 'selected' : '' ); ?>><?php esc_html_e( 'Categories & Tags', 'wpc-product-table' ); ?></option>
                                        <option value="on_sale" <?php echo esc_attr( $data['type'] === 'on_sale' ? 'selected' : '' ); ?>><?php esc_html_e( 'On Sale', 'wpc-product-table' ); ?></option>
                                        <option value="best_selling" <?php echo esc_attr( $data['type'] === 'best_selling' ? 'selected' : '' ); ?>><?php esc_html_e( 'Best Selling', 'wpc-product-table' ); ?></option>
										<?php
										$taxonomies = get_taxonomies( [ 'object_type' => [ 'product' ] ], 'objects' );

										foreach ( $taxonomies as $taxonomy ) {
											echo '<option value="' . esc_attr( $taxonomy->name ) . '" ' . ( $data['type'] === $taxonomy->name ? 'selected' : '' ) . '>' . esc_html( $taxonomy->label ) . '</option>';
										}
										?>
                                    </select>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_products">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Products', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <div class="wpcpt-product-search-wrapper">
                                        <input id=""
                                               class="wpcpt-product-search-input"
                                               name="wpcpt_configuration_include_products" type="hidden"
                                               value="<?php echo esc_attr( $data['include'] ); ?>"/>
                                        <select class="wc-product-search wpcpt-product-search" data-sortable="true"
                                                multiple="multiple"
                                                data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'wpc-product-table' ); ?>"
                                                data-action="woocommerce_json_search_products_and_variations">
											<?php
											$_product_ids = explode( ',', $data['include'] );

											foreach ( $_product_ids as $_product_id ) {
												$_product = wc_get_product( $_product_id );

												if ( $_product ) {
													echo '<option value="' . esc_attr( $_product_id ) . '" selected="selected">' . wp_kses_post( $_product->get_formatted_name() ) . '</option>';
												}
											}
											?>
                                        </select>
                                    </div>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_categories_tags">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Categories', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <div class="wpcpt-category-search-wrapper">
                                        <input id="" class="wpcpt-category-search-input"
                                               name="wpcpt_configuration_categories" type="hidden"
                                               value="<?php echo esc_attr( $data['categories'] ); ?>"/>
                                        <select class="wc-category-search wpcpt-category-search" multiple="multiple"
                                                data-placeholder="<?php esc_attr_e( 'Search for a category&hellip;', 'wpc-product-table' ); ?>">
											<?php
											$category_slugs = explode( ',', $data['categories'] );

											foreach ( $category_slugs as $category_slug ) {
												$category = get_term_by( 'slug', $category_slug, 'product_cat' );

												if ( $category ) {
													echo '<option value="' . esc_attr( $category_slug ) . '" selected="selected">' . wp_kses_post( $category->name ) . '</option>';
												}
											}
											?>
                                        </select>
                                    </div>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_categories_tags">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Tags', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input name="wpcpt_configuration_tags" type="text" style="width: 100%"
                                           placeholder="<?php esc_attr_e( 'Add some tags, split by a comma...', 'wpc-product-table' ); ?>"
                                           value="<?php echo esc_attr( $data['tags'] ); ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_terms">
                                <td class="wpcpt_configuration_th wpcpt_configuration_type_terms_label">
									<?php esc_html_e( 'Terms', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input name="wpcpt_configuration_terms" class="wpcpt_configuration_terms"
                                           type="hidden" style="width: 100%"
                                           value="<?php echo esc_attr( $data['terms'] ); ?>"/>
									<?php
									$terms = $data['terms'];

									if ( ! is_array( $terms ) ) {
										$terms = array_map( 'trim', explode( ',', $terms ) );
									}
									?>
                                    <select class="wpcpt_configuration_terms_select" multiple="multiple"
                                            data-<?php echo esc_attr( $data['type'] ); ?>="<?php echo esc_attr( implode( ',', $terms ) ); ?>">
										<?php
										if ( ! empty( $terms ) ) {
											foreach ( $terms as $t ) {
												if ( $term = get_term_by( 'slug', $t, $data['type'] ) ) {
													echo '<option value="' . esc_attr( $t ) . '" selected>' . esc_html( $term->name ) . '</option>';
												}
											}
										}
										?>
                                    </select>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_categories_tags">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Exclude products', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <div class="wpcpt-product-search-wrapper">
                                        <input id=""
                                               class="wpcpt-product-search-input"
                                               name="wpcpt_configuration_exclude_products" type="hidden"
                                               value="<?php echo esc_attr( $data['exclude'] ); ?>"/>
                                        <select class="wc-product-search wpcpt-product-search" data-sortable="true"
                                                multiple="multiple"
                                                data-placeholder="<?php esc_attr_e( 'Search for a product&hellip;', 'wpc-product-table' ); ?>"
                                                data-action="woocommerce_json_search_products_and_variations">
											<?php
											$_product_ids = explode( ',', $data['exclude'] );

											foreach ( $_product_ids as $_product_id ) {
												$_product = wc_get_product( $_product_id );

												if ( $_product ) {
													echo '<option value="' . esc_attr( $_product_id ) . '" selected="selected">' . wp_kses_post( $_product->get_formatted_name() ) . '</option>';
												}
											}
											?>
                                        </select>
                                    </div>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_categories_tags wpcpt_configuration_type_on_sale wpcpt_configuration_type_best_selling wpcpt_configuration_type_terms">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Limit', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input name="wpcpt_configuration_limit" type="number"
                                           value="<?php echo esc_attr( $data['limit'] ); ?>"
                                           placeholder="50"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_products wpcpt_configuration_type_categories_tags wpcpt_configuration_type_on_sale wpcpt_configuration_type_terms">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Order by', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <select name="wpcpt_configuration_orderby">
                                        <option value="default" <?php echo esc_attr( $data['orderby'] === 'default' ? 'selected' : '' ); ?>><?php esc_html_e( 'Default', 'wpc-product-table' ); ?></option>
                                        <option value="none" <?php echo esc_attr( $data['orderby'] === 'none' ? 'selected' : '' ); ?>><?php esc_html_e( 'None', 'wpc-product-table' ); ?></option>
                                        <option value="ID" <?php echo esc_attr( $data['orderby'] === 'ID' ? 'selected' : '' ); ?>><?php esc_html_e( 'ID', 'wpc-product-table' ); ?></option>
                                        <option value="name" <?php echo esc_attr( $data['orderby'] === 'name' ? 'selected' : '' ); ?>><?php esc_html_e( 'Name', 'wpc-product-table' ); ?></option>
                                        <option value="type" <?php echo esc_attr( $data['orderby'] === 'type' ? 'selected' : '' ); ?>><?php esc_html_e( 'Type', 'wpc-product-table' ); ?></option>
                                        <option value="rand" <?php echo esc_attr( $data['orderby'] === 'rand' ? 'selected' : '' ); ?>><?php esc_html_e( 'Rand', 'wpc-product-table' ); ?></option>
                                        <option value="date" <?php echo esc_attr( $data['orderby'] === 'date' ? 'selected' : '' ); ?>><?php esc_html_e( 'Date', 'wpc-product-table' ); ?></option>
                                        <option value="modified" <?php echo esc_attr( $data['orderby'] === 'modified' ? 'selected' : '' ); ?>><?php esc_html_e( 'Modified', 'wpc-product-table' ); ?></option>
                                    </select>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr wpcpt_configuration_type_row wpcpt_configuration_type_products wpcpt_configuration_type_categories_tags wpcpt_configuration_type_on_sale wpcpt_configuration_type_terms">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Order', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <select name="wpcpt_configuration_order">
                                        <option value="default" <?php echo esc_attr( $data['order'] === 'default' ? 'selected' : '' ); ?>><?php esc_html_e( 'Default', 'wpc-product-table' ); ?></option>
                                        <option value="desc" <?php echo esc_attr( $data['order'] === 'desc' ? 'selected' : '' ); ?>><?php esc_html_e( 'DESC', 'wpc-product-table' ); ?></option>
                                        <option value="asc" <?php echo esc_attr( $data['order'] === 'asc' ? 'selected' : '' ); ?>><?php esc_html_e( 'ASC', 'wpc-product-table' ); ?></option>
                                    </select>
                                </td>
                            </tr>
                        </table>
                    </div>
                    <div class="wpcpt_configuration wpcpt_configuration_design" style="display: none">
                        <table class="wpcpt_configuration_table">
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Style', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <select name="wpcpt_configuration_style">
                                        <option value="default" <?php echo esc_attr( $data['style'] === 'default' ? 'selected' : '' ); ?>><?php esc_html_e( 'Default', 'wpc-product-table' ); ?></option>
                                        <option value="01" <?php echo esc_attr( $data['style'] === '01' ? 'selected' : '' ); ?>><?php esc_html_e( '01 - Blue', 'wpc-product-table' ); ?></option>
                                        <option value="02" <?php echo esc_attr( $data['style'] === '02' ? 'selected' : '' ); ?>><?php esc_html_e( '02 - Green', 'wpc-product-table' ); ?></option>
                                        <option value="03" <?php echo esc_attr( $data['style'] === '03' ? 'selected' : '' ); ?>><?php esc_html_e( '03 - Orange', 'wpc-product-table' ); ?></option>
                                    </select>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Extra CSS classes', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_configuration_extra_classes"
                                           value="<?php echo esc_attr( $data['extra_classes'] ); ?>"/>
									<?php esc_html_e( 'Add extra CSS classes for the table, split by one space.', 'wpc-product-table' ); ?>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'Columns', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <div class="wpcpt-columns">
										<?php
										$saved_columns = get_post_meta( $post_id, 'columns', true );

										if ( is_array( $saved_columns ) && ( count( $saved_columns ) > 0 ) ) {
											foreach ( $saved_columns as $saved_column ) {
												self::column( $saved_column );
											}
										}
										?>
                                    </div>
									<?php self::new_column(); ?>
                                </td>
                            </tr>
                        </table>
                    </div>
                    <div class="wpcpt_configuration wpcpt_configuration_localization" style="display: none">
						<?php
						$localization = get_post_meta( $post_id, 'localization', true );
						?>
                        <table class="wpcpt_configuration_table">
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">

                                </td>
                                <td class="wpcpt_configuration_td">
									<?php esc_html_e( 'Read more about the below strings here https://datatables.net/reference/option/language', 'wpc-product-table' ); ?>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'decimal', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[decimal]"
                                           value="<?php echo esc_html( isset( $localization['decimal'] ) ? $localization['decimal'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['decimal']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'emptyTable', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[emptyTable]"
                                           value="<?php echo esc_html( isset( $localization['emptyTable'] ) ? $localization['emptyTable'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['emptyTable']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'info', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[info]"
                                           value="<?php echo esc_html( isset( $localization['info'] ) ? $localization['info'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['info']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'infoEmpty', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[infoEmpty]"
                                           value="<?php echo esc_html( isset( $localization['infoEmpty'] ) ? $localization['infoEmpty'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['infoEmpty']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'infoFiltered', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[infoFiltered]"
                                           value="<?php echo esc_html( isset( $localization['infoFiltered'] ) ? $localization['infoFiltered'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['infoFiltered']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'infoPostFix', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[infoPostFix]"
                                           value="<?php echo esc_html( isset( $localization['infoPostFix'] ) ? $localization['infoPostFix'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['infoPostFix']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'thousands', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[thousands]"
                                           value="<?php echo esc_html( isset( $localization['thousands'] ) ? $localization['thousands'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['thousands']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'lengthMenu', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[lengthMenu]"
                                           value="<?php echo esc_html( isset( $localization['lengthMenu'] ) ? $localization['lengthMenu'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['lengthMenu']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'loadingRecords', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[loadingRecords]"
                                           value="<?php echo esc_html( isset( $localization['loadingRecords'] ) ? $localization['loadingRecords'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['loadingRecords']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'processing', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[processing]"
                                           value="<?php echo esc_html( isset( $localization['processing'] ) ? $localization['processing'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['processing']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'search', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[search]"
                                           value="<?php echo esc_html( isset( $localization['search'] ) ? $localization['search'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['search']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'zeroRecords', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[zeroRecords]"
                                           value="<?php echo esc_html( isset( $localization['zeroRecords'] ) ? $localization['zeroRecords'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['zeroRecords']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'paginate:first', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[paginate][first]"
                                           value="<?php echo esc_html( isset( $localization['paginate']['first'] ) ? $localization['paginate']['first'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['paginate']['first']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'paginate:last', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[paginate][last]"
                                           value="<?php echo esc_html( isset( $localization['paginate']['last'] ) ? $localization['paginate']['last'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['paginate']['last']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'paginate:next', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[paginate][next]"
                                           value="<?php echo esc_html( isset( $localization['paginate']['next'] ) ? $localization['paginate']['next'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['paginate']['next']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'paginate:previous', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[paginate][previous]"
                                           value="<?php echo esc_html( isset( $localization['paginate']['previous'] ) ? $localization['paginate']['previous'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['paginate']['previous']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'aria:sortAscending', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[aria][sortAscending]"
                                           value="<?php echo esc_html( isset( $localization['aria']['sortAscending'] ) ? $localization['aria']['sortAscending'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['aria']['sortAscending']; ?>"/>
                                </td>
                            </tr>
                            <tr class="wpcpt_configuration_tr">
                                <td class="wpcpt_configuration_th">
									<?php esc_html_e( 'aria:sortDescending', 'wpc-product-table' ); ?>
                                </td>
                                <td class="wpcpt_configuration_td">
                                    <input type="text" name="wpcpt_localization[aria][sortDescending]"
                                           value="<?php echo esc_html( isset( $localization['aria']['sortDescending'] ) ? $localization['aria']['sortDescending'] : '' ); ?>"
                                           placeholder="<?php echo self::$localization['aria']['sortDescending']; ?>"/>
                                </td>
                            </tr>
                        </table>
                    </div>
					<?php
				}

				function column( $tab, $new = false ) {
					?>
                    <div class="wpcpt-column wpcpt-column-<?php echo $tab['type']; ?>">
                        <span class="button wpcpt-column-move"><?php esc_html_e( 'Move', 'wpc-product-table' ); ?></span>
                        <span class="wpcpt-column-label">#<?php echo esc_attr( $tab['type'] ); ?></span>
                        <select name="wpcpt_columns[orderable][]">
                            <option value="false" <?php echo esc_attr( $tab['orderable'] === 'false' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Orderable: No', 'wpc-product-table' ); ?></option>
                            <option value="true" <?php echo esc_attr( $tab['orderable'] === 'true' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Orderable: Yes', 'wpc-product-table' ); ?></option>
                        </select>
                        <select name="wpcpt_columns[align][]">
                            <option value="left" <?php echo esc_attr( $tab['align'] === 'left' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Align: Left', 'wpc-product-table' ); ?></option>
                            <option value="center" <?php echo esc_attr( $tab['align'] === 'center' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Align: Center', 'wpc-product-table' ); ?></option>
                            <option value="right" <?php echo esc_attr( $tab['align'] === 'right' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Align: Right', 'wpc-product-table' ); ?></option>
                        </select>
                        <select name="wpcpt_columns[valign][]">
                            <option value="top" <?php echo esc_attr( $tab['valign'] === 'top' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Vertical align: Top', 'wpc-product-table' ); ?></option>
                            <option value="middle" <?php echo esc_attr( $tab['valign'] === 'middle' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Vertical align: Middle', 'wpc-product-table' ); ?></option>
                            <option value="bottom" <?php echo esc_attr( $tab['valign'] === 'bottom' ? 'selected' : '' ); ?>><?php esc_attr_e( 'Vertical align: Bottom', 'wpc-product-table' ); ?></option>
                        </select>
                        <span class="button wpcpt-column-remove"><?php esc_html_e( 'Remove', 'wpc-product-table' ); ?></span>
                        <div class="wpcpt-column-line">
                            <input type="hidden"
                                   name="wpcpt_columns[type][]"
                                   value="<?php echo esc_attr( $tab['type'] ); ?>"/>
                            <input type="text"
                                   name="wpcpt_columns[title][]"
                                   placeholder="<?php echo esc_attr( $tab['type'] ); ?>"
                                   style="width: 100%"
                                   value="<?php echo esc_attr( $tab['title'] ); ?>"
                                   required/>
                            <input type="hidden"
                                   name="wpcpt_columns[content][]"
                                   value=""/>
                        </div>
                    </div>
					<?php
				}

				function add_column() {
					$type = isset( $_POST['type'] ) ? sanitize_key( $_POST['type'] ) : 'order';

					self::column( array(
						'editor'    => isset( $_POST['editor'] ) ? sanitize_key( $_POST['editor'] ) : '',
						'type'      => $type,
						'orderable' => 'false',
						'align'     => 'left',
						'valign'    => 'top',
						'title'     => self::$columns[ $type ],
						'content'   => ''
					), true );

					die();
				}

				function new_column() {
					?>
                    <div class="wpcpt-columns-new">
                        <select class="wpcpt-column-type">
							<?php
							foreach ( self::$columns as $key => $val ) {
								echo '<option value="' . $key . '" ' . ( $key === 'custom' ? 'disabled' : '' ) . '>' . $val . '</option>';
							}
							?>
                        </select>
                        <input type="button" class="button wpcpt-column-new"
                               value="<?php esc_attr_e( '+ Add new column', 'wpc-product-table' ); ?>"/>
                    </div>
					<?php
				}

				function table_columns( $columns ) {
					return array(
						'cb'              => $columns['cb'],
						'title'           => esc_html__( 'Title', 'wpc-product-table' ),
						'wpcpt_shortcode' => esc_html__( 'Shortcode', 'wpc-product-table' ),
						'wpcpt_desc'      => esc_html__( 'Description', 'wpc-product-table' ),
						'date'            => esc_html__( 'Date', 'wpc-product-table' ),
					);
				}

				function table_custom_column( $column, $postid ) {
					if ( $column === 'wpcpt_shortcode' ) {
						echo '<input type="text" class="wpcpt_shortcode_input" readonly value="[wpc_product_table id=&quot;' . $postid . '&quot; name=&quot;' . esc_attr( get_the_title( $postid ) ) . '&quot;]"/>';
					}

					if ( $column === 'wpcpt_desc' ) {
						echo get_the_excerpt( $postid );
					}
				}

				function save_product_table( $post_id ) {
					if ( isset( $_POST['wpcpt_configuration_type'] ) ) {
						update_post_meta( $post_id, 'type', sanitize_text_field( $_POST['wpcpt_configuration_type'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_categories'] ) ) {
						update_post_meta( $post_id, 'categories', sanitize_text_field( $_POST['wpcpt_configuration_categories'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_tags'] ) ) {
						update_post_meta( $post_id, 'tags', sanitize_text_field( $_POST['wpcpt_configuration_tags'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_terms'] ) ) {
						update_post_meta( $post_id, 'terms', sanitize_text_field( $_POST['wpcpt_configuration_terms'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_include_products'] ) ) {
						update_post_meta( $post_id, 'include', sanitize_text_field( $_POST['wpcpt_configuration_include_products'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_exclude_products'] ) ) {
						update_post_meta( $post_id, 'exclude', sanitize_text_field( $_POST['wpcpt_configuration_exclude_products'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_limit'] ) ) {
						update_post_meta( $post_id, 'limit', sanitize_key( $_POST['wpcpt_configuration_limit'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_orderby'] ) ) {
						update_post_meta( $post_id, 'orderby', sanitize_key( $_POST['wpcpt_configuration_orderby'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_order'] ) ) {
						update_post_meta( $post_id, 'order', sanitize_key( $_POST['wpcpt_configuration_order'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_style'] ) ) {
						update_post_meta( $post_id, 'style', sanitize_key( $_POST['wpcpt_configuration_style'] ) );
					}

					if ( isset( $_POST['wpcpt_configuration_extra_classes'] ) ) {
						update_post_meta( $post_id, 'extra_classes', self::sanitize_classes( $_POST['wpcpt_configuration_extra_classes'] ) );
					}

					if ( isset( $_POST['wpcpt_columns'] ) ) {
						update_post_meta( $post_id, 'columns', self::format_array( $_POST['wpcpt_columns'] ) );
					}

					if ( isset( $_POST['wpcpt_localization'] ) ) {
						update_post_meta( $post_id, 'localization', self::sanitize_array( $_POST['wpcpt_localization'] ) );
					}
				}

				function sanitize_array( $array ) {
					foreach ( $array as $key => &$value ) {
						if ( is_array( $value ) ) {
							$value = self::sanitize_array( $value );
						} else {
							$value = sanitize_text_field( $value );
						}
					}

					return $array;
				}

				function sanitize_classes( $classes ) {
					$sanitized = preg_replace( '/\s+/', ' ', trim( $classes ) );
					$sanitized = preg_replace( '/[^\sA-Za-z0-9_-]/', '', $sanitized );

					return $sanitized;
				}

				function format_array( $array ) {
					$formatted_array = array();

					foreach ( array_keys( $array ) as $field_key ) {
						foreach ( $array[ $field_key ] as $key => $value ) {
							$formatted_array[ $key ][ $field_key ] = wp_kses_post( $value );
						}
					}

					return $formatted_array;
				}

				function enqueue_scripts() {
					wp_enqueue_style( 'datatables', WPCPT_URI . 'assets/libs/datatables/datatables.min.css', array(), '1.10.22' );
					wp_enqueue_script( 'datatables', WPCPT_URI . 'assets/libs/datatables/datatables.min.js', array( 'jquery' ), '1.10.22', true );
					wp_enqueue_style( 'wpcpt-frontend', WPCPT_URI . 'assets/css/frontend.css', array(), WPCPT_VERSION );
					wp_enqueue_script( 'wpcpt-frontend', WPCPT_URI . 'assets/js/frontend.js', array( 'jquery' ), WPCPT_VERSION, true );
				}

				function admin_enqueue_scripts() {
					wp_enqueue_style( 'wpcpt-backend', WPCPT_URI . 'assets/css/backend.css', array( 'woocommerce_admin_styles' ), WPCPT_VERSION );
					wp_enqueue_script( 'wpcpt-backend', WPCPT_URI . 'assets/js/backend.js', array(
						'jquery',
						'wc-enhanced-select',
						'jquery-ui-sortable',
						'wp-color-picker',
						'selectWoo'
					) );
				}

				function search_term() {
					$return = array();

					$args = array(
						'taxonomy'   => sanitize_text_field( $_REQUEST['taxonomy'] ),
						'orderby'    => 'id',
						'order'      => 'ASC',
						'hide_empty' => false,
						'fields'     => 'all',
						'name__like' => sanitize_text_field( $_REQUEST['q'] ),
					);

					$terms = get_terms( $args );

					if ( count( $terms ) ) {
						foreach ( $terms as $term ) {
							$return[] = array( $term->slug, $term->name );
						}
					}

					wp_send_json( $return );
					wp_die();
				}
			}

			WPCleverWpcpt::instance();
		}
	}
}

if ( ! function_exists( 'wpcpt_notice_wc' ) ) {
	function wpcpt_notice_wc() {
		?>
        <div class="error">
            <p><strong>WPC Product Table</strong> requires WooCommerce version 3.0 or greater.</p>
        </div>
		<?php
	}
}
